package com.hai.service.impl;

import com.hai.common.exception.ErrorCode;
import com.hai.common.exception.ErrorHelp;
import com.hai.common.exception.SysCode;
import com.hai.common.utils.ResponseMsgUtil;
import com.hai.dao.HighCouponAgentCodeMapper;
import com.hai.dao.HighCouponAgentRelMapper;
import com.hai.entity.*;
import com.hai.service.HighCouponAgentService;
import com.hai.service.HighCouponCodeService;
import com.hai.service.HighCouponService;
import org.apache.commons.collections4.MapUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Auther: 胡锐
 * @Description:
 * @Date: 2021/4/20 23:02
 */
@Service("highCouponAgentService")
public class HighCouponAgentServiceImpl implements HighCouponAgentService {

    @Resource
    private HighCouponAgentRelMapper highCouponAgentRelMapper;

    @Resource
    private HighCouponAgentCodeMapper highCouponAgentCodeMapper;

    @Resource
    private HighCouponCodeService highCouponCodeService;

    @Resource
    private HighCouponService highCouponService;

    @Override
    public void assignCouponAgent(HighCouponAgentRel highCouponAgentRel) {

        // 查询卡券库存数量
        Map<String,Object> map = new HashMap<>();
        map.put("couponId", highCouponAgentRel.getCouponId());
        map.put("status", 1);
        map.put("isAssignAgent", false);
        List<HighCouponCode> codeList = highCouponCodeService.getCouponCodeList(map);

        if (highCouponAgentRel.getStockCount() > codeList.size()) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "无法分配，分配数量超过库存数量");
        }

        highCouponAgentRelMapper.insert(highCouponAgentRel);

        HighCouponAgentCode highCouponAgentCode;
        for (HighCouponCode code : codeList) {
            code.setAgentId(highCouponAgentRel.getAgentId());
            code.setIsAssignAgent(true);
            highCouponCodeService.updateCouponCode(code);

            highCouponAgentCode = new HighCouponAgentCode();
            highCouponAgentCode.setCouponAgentId(highCouponAgentRel.getId());
            highCouponAgentCode.setCouponId(code.getCouponId());
            highCouponAgentCode.setAgentId(highCouponAgentRel.getAgentId());
            highCouponAgentCode.setCouponCodeId(code.getId());
            highCouponAgentCode.setCouponCode(code.getSalesCode());
            highCouponAgentCode.setQrCode(code.getExt1());
            highCouponAgentCode.setStatus(1);
            highCouponAgentCode.setCreateTime(new Date());
            highCouponAgentCode.setOperatorId(highCouponAgentRel.getOperatorId());
            highCouponAgentCode.setOperatorName(highCouponAgentRel.getOperatorName());
            highCouponAgentCodeMapper.insert(highCouponAgentCode);
        }
    }

    @Override
    public HighCouponAgentRel getRelByCouponAgent(Long couponId, Long agentId) {
        HighCouponAgentRelExample example = new HighCouponAgentRelExample();
        example.createCriteria().andCouponIdEqualTo(couponId).andAgentIdEqualTo(agentId);
        List<HighCouponAgentRel> list = highCouponAgentRelMapper.selectByExample(example);
        if (list != null && list.size() > 0) {
            return list.get(0);
        }
        return null;
    }

    @Override
    public List<HighCouponAgentRel> getCouponAgentList(Map<String, Object> map) {
        HighCouponAgentRelExample example = new HighCouponAgentRelExample();
        HighCouponAgentRelExample.Criteria criteria = example.createCriteria();

        if (MapUtils.getLong(map, "couponId") != null) {
            criteria.andCouponIdEqualTo(MapUtils.getLong(map, "couponId"));
        }

        if (MapUtils.getLong(map, "agentId") != null) {
            criteria.andAgentIdEqualTo(MapUtils.getLong(map, "agentId"));
        }

        criteria.andStatusEqualTo(1);
        example.setOrderByClause("create_time desc");
        return highCouponAgentRelMapper.selectByExample(example);
    }

    @Override
    public List<HighCouponAgentCode> getCouponCodeList(Map<String, Object> map) {
        HighCouponAgentCodeExample example = new HighCouponAgentCodeExample();
        HighCouponAgentCodeExample.Criteria criteria = example.createCriteria();

        if (MapUtils.getLong(map, "couponAgentId") != null) {
            criteria.andCouponAgentIdEqualTo(MapUtils.getLong(map, "couponAgentId"));
        }

        if (MapUtils.getLong(map, "couponId") != null) {
            criteria.andCouponIdEqualTo(MapUtils.getLong(map, "couponId"));
        }

        if (MapUtils.getLong(map, "agentId") != null) {
            criteria.andAgentIdEqualTo(MapUtils.getLong(map, "agentId"));
        }

        if (MapUtils.getInteger(map, "status") != null) {
            criteria.andStatusEqualTo(MapUtils.getInteger(map, "status"));
        }

        example.setOrderByClause("create_time desc");
        return highCouponAgentCodeMapper.selectByExample(example);
    }

    @Override
    public HighCouponAgentCode getCodeById(Long couponAgentCodeId) {
        HighCouponAgentCodeExample example = new HighCouponAgentCodeExample();
        example.createCriteria().andCouponAgentIdEqualTo(couponAgentCodeId);
        List<HighCouponAgentCode> list = highCouponAgentCodeMapper.selectByExample(example);
        if (list.size() > 0) {
            return list.get(0);
        }
        return null;
    }

    @Override
    @Transactional(propagation= Propagation.REQUIRES_NEW)
    public Map<String, Object> generateCode(Long couponAgentCodeId) {
        // 查询卡券销售码
        HighCouponAgentCode couponAgentCode = getCodeById(couponAgentCodeId);
        if (couponAgentCode == null) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未找到销售码");
        }
        if (couponAgentCode.getStatus() != 1) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "销售码已被销售");
        }
        couponAgentCode.setStatus(2);  // 状态：1.待销售 2.未使用 3.已使用
        highCouponAgentCodeMapper.updateByPrimaryKey(couponAgentCode);

        HighCouponCode couponCode = highCouponCodeService.getCouponCodeById(couponAgentCode.getCouponCodeId());
        couponCode.setStatus(2);
        couponCode.setReceiveTime(new Date());
        highCouponCodeService.updateCouponCode(couponCode);

        Map<String, Object> map = new HashMap<>();
        map.put("couponInfo", highCouponService.getCouponById(couponAgentCode.getCouponId()));
        map.put("couponCode", couponCode);
        map.put("couponAgentCode", couponAgentCode);
        return map;
    }
}
